#!/usr/bin/env node

'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var program = _interopDefault(require('commander'));
var chalk = _interopDefault(require('chalk'));
var fileSize = _interopDefault(require('filesize'));
var path = require('path');
var prettyMs = _interopDefault(require('pretty-ms'));
var readPkg = _interopDefault(require('read-pkg'));
var slash = _interopDefault(require('slash'));
var __ = require('..');
var fs = require('fs');
var rollup = require('rollup');

var name = "bundlib";
var displayName = "Bundlib";
var version = "0.15.2";

function oneByOne(arr, callback) {
  let index = 0;
  const len = arr.length;
  const last = len - 1;

  const next = () => {
    if (index > last) {
      return;
    }

    callback(arr[index], index < last ? next : null);
    index++;
  };

  next();
}

function build(configs, callbacks) {
  if (callbacks.start) {
    callbacks.start();
  }

  const cache = {};
  oneByOne(configs, async (config, next) => {
    const {
      input,
      output
    } = config;
    const {
      file: outputFile,
      format
    } = output;
    const isModuleBuild = format === 'cjs' || format === 'es';
    const cacheKey = `${isModuleBuild ? 'module' : 'browser'}:${input}`;
    config.cache = cache[cacheKey];

    if (callbacks.buildStart) {
      callbacks.buildStart(input, outputFile);
    }

    try {
      const currentTime = Date.now();
      const buildResult = await rollup.rollup(config);
      config.cache = cache[cacheKey] = buildResult.cache;
      await buildResult.write(output);
      const {
        size
      } = fs.statSync(outputFile);
      const totalTime = Date.now() - currentTime;

      if (callbacks.buildEnd) {
        callbacks.buildEnd(outputFile, size, totalTime);
      }
    } catch (err) {
      if (callbacks.error) {
        callbacks.error(err);
      }

      if (callbacks.end) {
        callbacks.end();
      }

      throw err;
    }

    if (next) {
      return next();
    }

    if (callbacks.end) {
      callbacks.end();
    }
  });
}

function watch(configs, callbacks) {
  const {
    start,
    end,
    buildStart,
    buildEnd,
    error
  } = callbacks;

  const ERROR = error && (event => {
    error(event.error);
  });

  const table = {
    START: start && (() => {
      start();
    }),
    END: end && (() => {
      end();
    }),
    BUNDLE_START: buildStart && (event => {
      const {
        output: out
      } = event;
      const {
        length: len
      } = out;

      for (let i = 0; i < len; i++) {
        buildStart(event.input, out[i]);
      }
    }),
    BUNDLE_END: buildEnd && (event => {
      const {
        output: out
      } = event;
      const {
        length: len
      } = out;

      for (let i = 0; i < len; i++) {
        const stats = fs.statSync(out[i]);
        buildEnd(out[i], stats.size, event.duration || 0);
      }
    }),
    ERROR,
    FATAL: ERROR
  };
  const watcher = rollup.watch(configs);
  watcher.on('event', event => {
    const {
      code
    } = event;
    const handle = table[code];

    if (handle) {
      handle(event);
    }
  });
}

async function bundlib(cwd, options, callbacks, pkg) {
  const configs = await __.configsFromPkg(cwd, options, pkg);
  const {
    warn
  } = callbacks;
  const onwarn = warn ? warning => warn(warning) : () => null;
  configs.forEach(config => {
    config.onwarn = onwarn;
  });
  (options.watch ? watch : build)(configs, callbacks);
}

function create(name) {
  const method = console[name];
  return msg => method(chalk.cyan(msg));
}

const log = create('log');
const error = create('error');
function logError(err) {
  if (err.stack) {
    error(err.stack);
  }

  const tag = chalk.bold.bgWhite.red.inverse(' error ');
  error(`${tag} ${err.message || err}`);
}

const {
  bold
} = chalk;
const green = bold.green;
const yellow = bold.yellow;
const magenta = bold.magenta;

function prjInfo(name, ver) {
  const projName = green(name);
  const projVer = yellow(`v${ver}`);
  return `${projName} ${projVer}`;
}

if (!chalk.level && 'MINGW_CHOST' in process.env) {
  chalk.level = 1;
}

async function action(displayName, version, silent, options) {
  const cwd = process.cwd();
  let pkg;

  if (!silent) {
    const appInfo = prjInfo(displayName, version);
    log(`${appInfo}
`);
    const filename = yellow('package.json');
    log(`reading: ${filename}`);
    pkg = await readPkg({
      cwd,
      normalize: false
    });
    const {
      name: prjName,
      displayName: prjDispName,
      version: prjVer
    } = pkg;
    const prjInfoName = prjDispName || prjName;

    if (prjInfoName && prjVer) {
      const info = prjInfo(prjInfoName, prjVer);
      log(`building: ${info}
`);
    }
  }

  let buildIndex = 0;
  const callbacks = {
    error(err) {
      logError(err);
      process.exit(1);
    }

  };

  if (!silent) {
    Object.assign(callbacks, {
      buildEnd(filename, size, duration) {
        const tag = green.bgBlack.inverse(' built ');
        const path$1 = yellow(slash(path.relative(cwd, filename)));
        const colorSize = magenta(fileSize(size));
        const colorTime = magenta(prettyMs(duration, {
          secondsDecimalDigits: 2
        }));
        log(`${tag} ${path$1} ( ${colorSize} in ${colorTime} )`);
      },

      error(err) {
        logError(err);
      },

      warn(warning) {
        let message = warning;

        if (typeof message === 'object') {
          const {
            plugin,
            message: msg
          } = message;
          message = msg;

          if (plugin) {
            message = `(plugin ${magenta(plugin)}) ${message}`;
          }
        }

        const tag = magenta('warning:');
        log(`${tag} ${message}`);
      }

    });

    if (options.watch) {
      Object.assign(callbacks, {
        start() {
          if (buildIndex) {
            log(`rebuilding...
`);
          }

          buildIndex++;
        },

        end() {
          log(`
waiting for changes...`);
        }

      });
    }
  }

  try {
    await bundlib(cwd, options, callbacks, pkg);
  } catch (err) {
    logError(err);
    process.exit(1);
  }
}

program.version(version, '-v, --version').name(name).option('-d, --dev', 'create development builds').option('-w, --watch', 'run bundlib in watch mode').option('-s, --silent', 'prevent messages from showing in the console').action(async () => {
  const {
    dev,
    watch,
    silent
  } = program;
  await action(displayName, version, silent, {
    dev,
    watch
  });
}).parseAsync(process.argv);
//# sourceMappingURL=bundlib-cli.js.map
